/*********************************************************************************

 ** Universidade Nova de Lisboa - Faculdade de Ciências e Tecnologia

 **

 ** Protocolo XORi - flatbeacon_neigb_table.cc

 **

 ** Autores:

 ** André Garrido

 ** Rodolfo Oliveira

 **

 ** 1 de Março de 2010

** **********************************************************************************/

#include "flatbeacon_neigb_table.h"
#include <ip.h>
#include <map>
#include <vector>
#include <math.h>


flatbeacon_neigb_table::flatbeacon_neigb_table(flatbeacon* flat, nsaddr_t source) {

	bk_.clear();
	flat_= flat;
	source_= source;
}


///Prints the routing table

void flatbeacon_neigb_table::print(nsaddr_t node,int codebit){

	printf("\nneighbors table of node: %i\n",node);


	for(int i=0;i<codebit;i++)
	{
		printf("\nNEIGHBOR OF BUCKET: %d\n",i);
		for(array_nodes::iterator it= bk_[i].begin(); it!= bk_[i].end();it++)
		{
			flatbeacon_neib_node* nb_bucket= *it;
			printf("dest: %i src: %i  num_hops:%i  msg_type: %i  col: %i   line: %i   id:%i\n",nb_bucket->dest_addr_,nb_bucket->next_hop_addr_,
					nb_bucket->num_hops_,nb_bucket->type_msg_,nb_bucket->col_,nb_bucket->line_,nb_bucket->id_);

		}
	}
}


///Clear the routing table
void flatbeacon_neigb_table::clear(){

	bk_.clear();
}


///Adds a new entry in the routing table
void flatbeacon_neigb_table::add_entry(nsaddr_t dest,nsaddr_t next_hop,nsaddr_t orig,int code, int num_hop,
		int type_msg,int line, int col,int id,bool for_query){



	char node_dest[code];
	char node_qry[code];

	int i  , k ,k1, mask;

	  for( i =(code-1) ; i >= 0 ; i--)
	  {
		 mask = 1 << i;
		 k = orig & mask;
		 k1= dest & mask;
		 if( k == 0)
		   node_qry[i]='0';
		 else
			node_qry[i]='1';

		 if(k1==0)
			 node_dest[i]='0';
		 else
			 node_dest[i]='1';
	  }

	flatbeacon_neib_node* nb_bucket= (flatbeacon_neib_node*) new flatbeacon_neib_node(code);

	nb_bucket->dest_addr_=dest;
	nb_bucket->next_hop_addr_= next_hop;
	nb_bucket->neig_table= this;
	nb_bucket->num_hops_= num_hop;
	nb_bucket->type_msg_= type_msg;
	nb_bucket->line_= line;
	nb_bucket->col_= col;
	nb_bucket->id_= id;
	nb_bucket->for_query= for_query;

	bool is_neigb= true;


	for(i=(code-1);i>=0;i--)
	{
		if(node_qry[i]!=node_dest[i])
		{

			for(std::vector<flatbeacon_neib_node*>::iterator it= bk_[i].begin(); it!= bk_[i].end();it++){

				flatbeacon_neib_node* nb_bucket2= *it;
				if(nb_bucket2->nb_dest_addr()==nb_bucket->nb_dest_addr() ){

					is_neigb= false;

					if(nb_bucket->for_query != nb_bucket2->for_query)
					{
						nb_bucket2->for_query= nb_bucket->for_query;
					}

				}
			}


			if(nb_bucket->dest_addr_== orig || nb_bucket->next_hop_addr_ == orig)
			{
				is_neigb= false;
			}

			if(is_neigb)
			{

				double interval = NODE_TIMEOUT;
				nb_bucket->timer.sched(interval);
				bk_[i].push_back(nb_bucket);


			} else {

				free(nb_bucket);
			}
			break;


		}
	}
}


///Adds a new entry from the RESPONSE
void flatbeacon_neigb_table::add_entry_resp(nsaddr_t dest,nsaddr_t next_hop,nsaddr_t orig,int code, int num_hop,
		bool query,int type_msg,int line, int col,int id,int line_q,int col_q,int type_q,int id_q, bool for_query)
{



	char node_dest[code];
	char node_qry[code];

	int i  , k ,k1, mask;

	  for( i =(code-1) ; i >= 0 ; i--)
	  {
		 mask = 1 << i;
		 k = orig & mask;
		 k1= dest & mask;
		 if( k == 0)
		   node_qry[i]='0';
		 else
			node_qry[i]='1';

		 if(k1==0)
			 node_dest[i]='0';
		 else
			 node_dest[i]='1';
	  }

	flatbeacon_neib_node* nb_bucket= (flatbeacon_neib_node*) new flatbeacon_neib_node(code);

	nb_bucket->dest_addr_=dest;
	nb_bucket->next_hop_addr_= next_hop;
	nb_bucket->neig_table= this;
	nb_bucket->num_hops_= num_hop;
	nb_bucket->type_msg_= type_msg;
	nb_bucket->line_= line;
	nb_bucket->col_= col;
	nb_bucket->id_= id;
	nb_bucket->for_query= for_query;

	bool is_neigb= true;


	for(i=(code-1);i>=0;i--)
	{
		if(node_qry[i]!=node_dest[i])
		{

			for(std::vector<flatbeacon_neib_node*>::iterator it= bk_[i].begin(); it!= bk_[i].end();it++){

				flatbeacon_neib_node* nb_bucket2= *it;
				if(nb_bucket2->nb_dest_addr()==nb_bucket->nb_dest_addr() || nb_bucket->dest_addr_ == source_){

					is_neigb= false;

				}
			}

			if(is_neigb)
			{

				if(flat_->cache_->confirm_node(line_q,col_q,type_q,id_q))
				{

					double interval = NODE_TIMEOUT;
					nb_bucket->timer.sched(interval);
					bk_[i].push_back(nb_bucket);

					///////SEND QUERY TO THE NEW NEIB//////////

					if(query)
					{
						flat_->send_query_new(nb_bucket->dest_addr_,nb_bucket->next_hop_addr_);
					}

				}else {

					free(nb_bucket);
				}



			} else {

				free(nb_bucket);
			}
			break;


		}
	}
}


///Searches which buckets are below K.
bool flatbeacon_neigb_table::buck_query(int k,int code,vector<int>* bucks)
{
	std::vector<int> buck;

	bool resul=false;

	for(int i=0;i<code;i++)
	{
		if(bk_[i].size()<k)
		{
			buck.push_back(i);
			resul= true;

		}
	}
	*bucks= buck;
	return resul;
}

///Search which nodes are suited for the bucktes which are sent from a neigbour.
bool flatbeacon_neigb_table::search_nodes_bucks(nsaddr_t src, vector<int> buckets,int code,vector<int>* resul_find_neib,
		int* count, vector<int>* resul_num_hop,vector<int>* resul_cols,vector<int>* resul_lines,vector<int>* resul_type,
		vector<int>* result_id)
{


	std::vector<int> vec_nodes;
	std::vector<int> vec_hop;
	std::vector<int> lines;
	std::vector<int> cols;
	std::vector<int> type;
	std::vector<int> ids;
	bool find_neib= false;

	char node_xor[code];


	int mask,k;

	for(int i=0; i<buckets.size(); i++)
	{
		for(int n=0;n<code;n++)
		{
			for(array_nodes::iterator it=bk_[n].begin();it!=bk_[n].end();it++)
			{
				flatbeacon_neib_node* nb_node= *it;
				if(nb_node->dest_addr_!=src)
				{
					int xor_resul= nb_node->dest_addr_ ^ src;

					for( int t =(code-1) ; t >= 0 ; t--)
					{
						 mask = 1 << t;
						 k = xor_resul & mask;
						 if( k == 0)
							 node_xor[t]='0';
						 else
							 node_xor[t]='1';
					}

					int count_xor=0;
					for(int y=(code-1);y>=0;y--)
					{
						if(node_xor[y]=='0')
						{
							count_xor++;
						} else
							break;
					}
					if(((code-1)-count_xor)==buckets[i])
					{
						find_neib= true;

						//printf("\nsrc: %d  neib: %d  buck: %d\n",src,nb_node->dest_addr_,buckets[i]);

						vec_nodes.push_back(nb_node->dest_addr_);
						vec_hop.push_back(nb_node->num_hops_);
						cols.push_back(nb_node->col_);
						lines.push_back(nb_node->line_);
						type.push_back(nb_node->type_msg_);
						ids.push_back(nb_node->id_);


					}
				}

			}
		}
	}

	*resul_find_neib= vec_nodes;
	*resul_num_hop= vec_hop;
	*resul_lines= lines;
	*resul_cols= cols;
	*resul_type= type;
	*result_id= ids;
	return find_neib;


}

///Inserts the nodes which are sent in the RESPONSE
void flatbeacon_neigb_table::insert_neib_query(nsaddr_t src, std::vector<int> neibs,nsaddr_t orig,int code,
		vector<int> neib_hop,int num_hops,bool query,int type_msg,int line,int col, int id,
		vector<int> lines, vector<int> cols, vector<int> types, vector<int> ids)
{

	for(int i=0; i<neibs.size(); i++)
	{
		add_entry_resp(neibs[i],src,orig,code,neib_hop[i]+num_hops, query,type_msg,line,col,id,lines[i],cols[i],types[i],
				ids[i],false);
	}


}

///Inserts the nodes which are sent in the message which contains the routing table
void flatbeacon_neigb_table::insert_neib_bgl(nsaddr_t src,std::vector<int> neibs,nsaddr_t orig,int code,vector<int> neib_hop,int num_hops,
		vector<int> lines, vector<int> cols, vector<int> types, vector<int> ids)
{

	for(int i= 0; i<neibs.size(); i++)
	{

		add_entry_resp(neibs[i],src,orig,code,neib_hop[i]+num_hops, false ,types[i],lines[i],cols[i],ids[i],lines[i],cols[i],types[i],
				ids[i],false);
	}

}

///Remove a entry in the routing table
void flatbeacon_neigb_table::rm_entry(nsaddr_t* src,int code)
{


	//printf("\nREMOVE NODE DA TABELA %d\n",flat_->ra_addr);
	//printf("clock: %f\n",Scheduler::instance().clock());
	for(int n=0;n<code;n++)
	{
		for(array_nodes::iterator it=bk_[n].begin();it!=bk_[n].end();it++)
		{
			flatbeacon_neib_node* nb_node= *it;
			if(nb_node->dest_addr_==*src || nb_node->next_hop_addr_==*src)
			{
				//printf("\nFOI REMOVIDO O NODE: %d\n",*src);
				nb_node->timer.force_cancel();
				it= bk_[n].erase(it);
				it--;
			}

		}

	}
}


///Update a node timer.
void flatbeacon_neigb_table::update_timer_neib(nsaddr_t src,int code)
{
	for(int n=0;n<code;n++)
	{
		for(array_nodes::iterator it=bk_[n].begin();it!=bk_[n].end();it++)
		{
			flatbeacon_neib_node* nb_node= *it;
			if(nb_node->dest_addr_==src)
			{
				double interval= NODE_TIMEOUT;
				nb_node->timer.resched(interval);

			}
		}
	}
}











