/*********************************************************************************

 ** Universidade Nova de Lisboa - Faculdade de Ciências e Tecnologia

 **

 ** Protocolo XORi - flatbeacon_query_pkt.h

 **

 ** Autores:

 ** André Garrido

 ** Rodolfo Oliveira

 **

 ** 1 de Março de 2010

** **********************************************************************************/

#ifndef __FLATBEACON_QUERY_PKT_H__
#define __FLATBEACON_QUERY_PKT_H__
#include <packet.h>
#include <vector>


#define HDR_FLATBEACON_QUERY_PKT(p) (hdr_flatbeacon_query_pkt::access(p))


/// MESSAGE's TYPES
#define PT_QRY 1
#define PT_REPLY_QUERY 2
#define PT_TAB_BGL 3

typedef struct hdr_flatbeacon_query_pkt {

	 u_int8_t   qry_type_;   //type of the query
     nsaddr_t   pkt_src_;     // Node which originated this packet
     nsaddr_t	pkt_hop_;	 // NOde which is sending the packet
     nsaddr_t	pkt_dest_;	// the destination node
     u_int8_t	num_hop_;   // number of hop's
     vector<int> buckets_;  // buckets/nodes list
     vector<int> buck_hop_; //hops list
     vector<int> line_;     // lines list
     vector<int> col_;      // cols list
     vector<int> type_;		// types list
     vector<int> ids_;		// ids	list
     u_int8_t	id_;		// identification number




     inline		u_int8_t& 	qry_tyre()		{return qry_type_; }
     inline     nsaddr_t&   pkt_src()     { return pkt_src_; }
     inline		nsaddr_t&	pkt_hop()	{ return pkt_hop_; }
     inline		nsaddr_t&	pkt_dest()	{ return pkt_dest_; }
     inline		u_int8_t&	num_hop()	{ return num_hop_;}
     inline		int&	buckets(int i)	{return buckets_[i]; }
     inline		int&	buck_hop(int i)	{return buck_hop_[i]; }
     inline		int&	lines(int i)	{return line_[i]; }
     inline		int&	cols(int i)		{return col_[i]; }
     inline		int&	types(int i)	{return type_[i]; }
     inline		u_int8_t&	qry_id()	{return id_;}
     inline		int&	qry_dis(int i)		{return ids_[i];}



     static int offset_;
     inline static int& offset() { return offset_; }
     inline static hdr_flatbeacon_query_pkt* access(const Packet* p) {
         return (hdr_flatbeacon_query_pkt*)p->access(offset_);
     }

     inline int& size() {
		int sz = 0;
		sz = sizeof(qry_type_)+sizeof(pkt_src_)+sizeof(pkt_hop_)+sizeof(buckets_)+sizeof(pkt_dest_)+ sizeof(num_hop_)+
				sizeof(buck_hop_)+sizeof(id_)+ sizeof(col_)+sizeof(line_)+sizeof(type_)+sizeof(ids_);
		assert (sz >= 0);
		return sz;
     }


}hdr_flatbeacon_query_pkt;


#endif /* FLATBEACON_QUERY_PKT_H_ */
