/*////////////////////////////////////////////////////////////////////////
//									//
//  TITLE: p2p_packets.h - P2P packets					//
//									//
//									//
//  FUNCTION: defines all communication packets used by nodes 		//
//									//
//									//
//  AUTHOR: Rodolfo Oliveira - June 2004				//
//	    rado@uninova.pt						//
//									//
//  New University of Lisbon / FCT - All rights reserved		//
//									//
////////////////////////////////////////////////////////////////////////*/
#ifndef p2p_packet_h
#define p2p_packet_h

#include <packet.h>


// P2P header accessor macros
#define HDR_P2P_BC(p) ((hdr_p2p_bc *)hdr_p2p::access(p))
/*#define HDR_P2P_SEARCH(p) ((hdr_p2p_search *)hdr_search::access(p))
#define HDR_P2P_FOUND(p) ((hdr_p2p_found *)hdr_found::access(p))
*/


// P2P packet types
#define P2P_PT_BC         1
/*#define P2P_PT_SEARCH     2
#define P2P_PT_FOUND      3
*/

////////////////////////////////////////////////////////////////////////////////7
//
//  Clustering Beacon Definition
//
/////////////////////////////////////////////////////////////////////////////////

// General P2P header
typedef struct hdr_p2p {
  u_int8_t p2p_type;
  double send_time;

  // Header access methods
  static int offset_; // required by PacketHeaderManager
  inline static int& offset() { return offset_; }

  inline static hdr_p2p* access(const Packet* p) {
    return (hdr_p2p*) p->access(offset_);
  }
} hdr_p2p;

// BEACON message header
typedef struct hdr_p2p_bc {
  u_int8_t   bc_type;
  u_int32_t  bc_seq_num; // beacon's sequence neumber
  u_int16_t  bc_associativity; // major V
  nsaddr_t   bc_gl_addr; // epson

    u_int8_t   num_elems;//numero de elementos na lista de BGL's vizinhos p/ faazer cluster 2.5 hop



  inline int size() {
     int sz = 0;
     sz = sizeof(bc_seq_num)+sizeof(bc_associativity)+sizeof(bc_gl_addr)+sizeof(bc_type)+sizeof(num_elems);
     assert (sz >= 0);
     return sz;
  }
}hdr_p2p_bc;

// For size calculation of header-space reservation
union hdr_all_p2p {
  hdr_p2p             p2p_general;
  hdr_p2p_bc          p2p_bc;
};

#endif //p2p_packet_h
